@@ -90,7 +90,7 @@ runCohortBasedBinaryNonAggTest <- function(connection, cdmDatabaseSchema, ohdsiD
   tempTable <- substr(cohortTable, 1, 1) == "#"
   covs <- getDbCovariateData(
     connection = connection,
-    oracleTempSchema = getOption("sqlRenderTempEmulationSchema"),
+    tempEmulationSchema = getOption("sqlRenderTempEmulationSchema"),
     cdmDatabaseSchema = cdmDatabaseSchema,
     cohortDatabaseSchema = ohdsiDatabaseSchema,
     cohortTableIsTemp = tempTable,
@@ -132,7 +132,7 @@ runCohortBasedBinaryAggTest <- function(connection, cdmDatabaseSchema, ohdsiData
   tempTable <- substr(cohortTable, 1, 1) == "#"
   covs <- getDbCovariateData(
     connection = connection,
-    oracleTempSchema = getOption("sqlRenderTempEmulationSchema"),
+    tempEmulationSchema = getOption("sqlRenderTempEmulationSchema"),
     cdmDatabaseSchema = cdmDatabaseSchema,
     cohortDatabaseSchema = ohdsiDatabaseSchema,
     cohortTableIsTemp = tempTable,
@@ -175,7 +175,7 @@ runCohortBasedBinaryNonAggTemporalTest <- function(connection, cdmDatabaseSchema
   tempTable <- substr(cohortTable, 1, 1) == "#"
   covs <- getDbCovariateData(
     connection = connection,
-    oracleTempSchema = getOption("sqlRenderTempEmulationSchema"),
+    tempEmulationSchema = getOption("sqlRenderTempEmulationSchema"),
     cdmDatabaseSchema = cdmDatabaseSchema,
     cohortDatabaseSchema = ohdsiDatabaseSchema,
     cohortTableIsTemp = tempTable,
@@ -219,7 +219,7 @@ runCohortBasedBinaryAggTemporalTest <- function(connection, cdmDatabaseSchema, o
   tempTable <- substr(cohortTable, 1, 1) == "#"
   covs <- getDbCovariateData(
     connection = connection,
-    oracleTempSchema = getOption("sqlRenderTempEmulationSchema"),
+    tempEmulationSchema = getOption("sqlRenderTempEmulationSchema"),
     cdmDatabaseSchema = cdmDatabaseSchema,
     cohortDatabaseSchema = ohdsiDatabaseSchema,
     cohortTableIsTemp = tempTable,
@@ -266,7 +266,7 @@ runCohortBasedCountsNonAggTest <- function(connection, cdmDatabaseSchema, ohdsiD
   tempTable <- substr(cohortTable, 1, 1) == "#"
   covs <- getDbCovariateData(
     connection = connection,
-    oracleTempSchema = getOption("sqlRenderTempEmulationSchema"),
+    tempEmulationSchema = getOption("sqlRenderTempEmulationSchema"),
     cdmDatabaseSchema = cdmDatabaseSchema,
     cohortDatabaseSchema = ohdsiDatabaseSchema,
     cohortTable = cohortTable,
@@ -310,7 +310,7 @@ runCohortBasedCountsAggTest <- function(connection, cdmDatabaseSchema, ohdsiData
   tempTable <- substr(cohortTable, 1, 1) == "#"
   covs <- getDbCovariateData(
     connection = connection,
-    oracleTempSchema = getOption("sqlRenderTempEmulationSchema"),
+    tempEmulationSchema = getOption("sqlRenderTempEmulationSchema"),
     cdmDatabaseSchema = cdmDatabaseSchema,
     cohortDatabaseSchema = ohdsiDatabaseSchema,
     cohortTableIsTemp = tempTable,
@@ -356,7 +356,7 @@ runCohortBasedCountsNonAggTemporalTest <- function(connection, cdmDatabaseSchema
   tempTable <- substr(cohortTable, 1, 1) == "#"
   covs <- getDbCovariateData(
     connection = connection,
-    oracleTempSchema = getOption("sqlRenderTempEmulationSchema"),
+    tempEmulationSchema = getOption("sqlRenderTempEmulationSchema"),
     cdmDatabaseSchema = cdmDatabaseSchema,
     cohortDatabaseSchema = ohdsiDatabaseSchema,
     cohortTableIsTemp = tempTable,
@@ -401,7 +401,7 @@ runCohortBasedCountsAggTemporalTest <- function(connection, cdmDatabaseSchema, o
   tempTable <- substr(cohortTable, 1, 1) == "#"
   covs <- getDbCovariateData(
     connection = connection,
-    oracleTempSchema = getOption("sqlRenderTempEmulationSchema"),
+    tempEmulationSchema = getOption("sqlRenderTempEmulationSchema"),
     cdmDatabaseSchema = cdmDatabaseSchema,
     cohortDatabaseSchema = ohdsiDatabaseSchema,
     cohortTable = cohortTable,
@@ -450,7 +450,7 @@ runCohortBasedCountsAggMultiCohortTest <- function(connection, cdmDatabaseSchema
   tempTable <- substr(cohortTable, 1, 1) == "#"
   covs <- getDbCovariateData(
     connection = connection,
-    oracleTempSchema = getOption("sqlRenderTempEmulationSchema"),
+    tempEmulationSchema = getOption("sqlRenderTempEmulationSchema"),
     cdmDatabaseSchema = cdmDatabaseSchema,
     cohortDatabaseSchema = ohdsiDatabaseSchema,
     cohortTableIsTemp = tempTable,
@@ -486,6 +486,7 @@ test_that("Cohort-based covariates: binary, non-aggregated on Eunomia", {
 })
 
 test_that("Cohort-based covariates: binary, aggregated on Eunomia", {
+  skip_on_cran()
   skip_if_not(dbms == "sqlite" && exists("eunomiaConnection"))
   runCohortBasedBinaryAggTest(
     connection = eunomiaConnection,
@@ -496,6 +497,7 @@ test_that("Cohort-based covariates: binary, aggregated on Eunomia", {
 })
 
 test_that("Cohort-based covariates: binary, non-aggregated, temporal on Eunomia", {
+  skip_on_cran()
   skip_if_not(dbms == "sqlite" && exists("eunomiaConnection"))
   runCohortBasedBinaryNonAggTemporalTest(
     connection = eunomiaConnection,
@@ -506,6 +508,7 @@ test_that("Cohort-based covariates: binary, non-aggregated, temporal on Eunomia"
 })
 
 test_that("Cohort-based covariates: binary, aggregated, temporal on Eunomia", {
+  skip_on_cran()
   skip_if_not(dbms == "sqlite" && exists("eunomiaConnection"))
   runCohortBasedBinaryAggTemporalTest(
     connection = eunomiaConnection,
@@ -516,6 +519,7 @@ test_that("Cohort-based covariates: binary, aggregated, temporal on Eunomia", {
 })
 
 test_that("Cohort-based covariates: counts, non-aggregated on Eunomia", {
+  skip_on_cran()
   skip_if_not(dbms == "sqlite" && exists("eunomiaConnection"))
   runCohortBasedCountsNonAggTest(
     connection = eunomiaConnection,
@@ -526,6 +530,7 @@ test_that("Cohort-based covariates: counts, non-aggregated on Eunomia", {
 })
 
 test_that("Cohort-based covariates: counts, aggregated on Eunomia", {
+  skip_on_cran()
   skip_if_not(dbms == "sqlite" && exists("eunomiaConnection"))
   runCohortBasedCountsAggTest(
     connection = eunomiaConnection,
@@ -536,6 +541,7 @@ test_that("Cohort-based covariates: counts, aggregated on Eunomia", {
 })
 
 test_that("Cohort-based covariates: counts, non-aggregated, temporal on Eunomia", {
+  skip_on_cran()
   skip_if_not(dbms == "sqlite" && exists("eunomiaConnection"))
   runCohortBasedCountsNonAggTemporalTest(
     connection = eunomiaConnection,
@@ -546,6 +552,7 @@ test_that("Cohort-based covariates: counts, non-aggregated, temporal on Eunomia"
 })
 
 test_that("Cohort-based covariates: counts, aggregated, temporal on Eunomia", {
+  skip_on_cran()
   skip_if_not(dbms == "sqlite" && exists("eunomiaConnection"))
   runCohortBasedCountsAggTemporalTest(
     connection = eunomiaConnection,
@@ -556,6 +563,7 @@ test_that("Cohort-based covariates: counts, aggregated, temporal on Eunomia", {
 })
 
 test_that("Cohort-based covariates: counts, aggregated, using multiple cohort IDs on Eunomia", {
+  skip_on_cran()
   skip_if_not(dbms == "sqlite" && exists("eunomiaConnection"))
   runCohortBasedCountsAggMultiCohortTest(
     connection = eunomiaConnection,
